<?php
/*
Plugin Name: I18N Search
Description: Search (I18N enabled!)
Version: 1.0
Author: Martin Vlcek
Author URI: http://mvlcek.bplaced.net

Public functions:
  return_i18n_search_results($params)
      returns an array of search results, each with the attributes url, parent, title, 
      date (UNIX timestamp), excerpt (if $excerptlength > 0), content (if $excerptlength < 0), score.
      The params are described on the configuration page.
  return_i18n_tags()
      returns an ordered array, where the key is the tag and the value is an array of
      urls (slugs) using that tag. Tags starting with "_" are ignored. 

Display functions:
  get_i18n_search_results($params)
      outputs the search results. params is an associative array and includes keys like 'tags'
      (to search for), 'words' (to search for), 'max' (number of results to display), ...
      If no tags or words are given they are taken from the request parameters 'tags' and 'words'
      A param 'numWords' < 0 will output the whole content of the page.
  get_i18n_tags($params)
      outputs the ordered tags in spans with class "tag" and a font-size proportional to the 
      percentage of pages using it (within the percentage range specified by the $params['minTagSize']
      and $params['maxTagSize']).
      Tags starting with "_" are ignored.
  get_i18n_search_form($params)
      outputs a search form. $params['slug'] is the URL of the result page.
*/

# get correct id for plugin
$thisfile = basename(__FILE__, ".php");

define('I18N_WORD_INDEX', 'i18n_word_index.txt'); 
define('I18N_TAG_INDEX', 'i18n_tag_index.txt');
define('I18N_DATE_INDEX', 'i18n_date_index.txt');
define('I18N_SEARCH_SETTINGS_FILE', 'i18n_search_settings.xml');

# for sorting results:
define('I18N_CONTENT_WEIGHT', 1);
define('I18N_TITLE_WEIGHT', 5);
define('I18N_TAG_WEIGHT', 10);
define('I18N_MAX_RESULTS', 10);
define('I18N_NUM_WORDS', 30);
define('I18N_MIN_TAG_SIZE', 100);
define('I18N_MAX_TAG_SIZE', 250);

# register plugin
register_plugin(
	$thisfile, 
	'I18N Search', 	
	'1.0', 		
	'Martin Vlcek',
	'http://mvlcek.bplaced.net', 
	'Search (I18N enabled!)',
	'plugins',
	'i18n_search_configure'  
);

# load i18n texts
require_once(GSPLUGINPATH.'i18n_common/common.php');
if (function_exists('i18n_load_texts')) {
  i18n_load_texts('i18n_search');
} else {
  i18n_merge('i18n_search', substr($LANG,0,2));
  i18n_merge('i18n_search', 'en');
}

# activate filter
add_action('changedata-save', 'i18n_clear_search_index'); 
add_action('page-delete', 'i18n_clear_search_index'); // GetSimple 3.0+
add_action('plugins-sidebar', 'createSideMenu', array($thisfile, i18n_r('i18n_search/CONFIGURE'))); 
add_filter('content','i18n_search_replace');

function i18n_clear_search_index() {
  if (file_exists(GSDATAOTHERPATH . I18N_WORD_INDEX)) unlink(GSDATAOTHERPATH . I18N_WORD_INDEX);
  if (file_exists(GSDATAOTHERPATH . I18N_TAG_INDEX)) unlink(GSDATAOTHERPATH . I18N_TAG_INDEX);
  if (file_exists(GSDATAOTHERPATH . I18N_DATE_INDEX)) unlink(GSDATAOTHERPATH . I18N_DATE_INDEX);
}

function i18n_create_search_index() {
  include(GSPLUGINPATH.'i18n_search/index.php');
}   

function &i18n_search($tags=null, $words=null) {
  $results = array();
  include(GSPLUGINPATH.'i18n_search/search.php');
  return $results;
}

function return_i18n_number_of_results($tags=null, $words=null) {
  $results = i18n_search($tags, $words);
  return $results ? count($results) : 0;
}

function i18n_is_word($line, $word, $ismb) {
  if (($ismb && mb_strlen($word, 'UTF-8') <= 3) || (!$ismb && strlen($word) <= 3)) {
    return strncmp($line, $word.' ', strlen($word)+1) == 0;
  } else {
    return strncmp($line, $word, strlen($word)) == 0;
  }
}

function return_i18n_tags() {
  if (!file_exists(GSDATAOTHERPATH . I18N_WORD_INDEX)) i18n_create_search_index();
  $tags = array();
  $f = fopen(GSDATAOTHERPATH . I18N_TAG_INDEX, "r");
  while (($line = fgets($f)) !== false) {
    $items = preg_split("/\s+/", trim($line));
    if (substr($items[0],0,1) != '_') {
      $tag = preg_replace("/_+/", " ", $items[0]);
      $urls = array_slice($items,1);
      $tags[$tag] = $urls;
    }
  }
  fclose($f);
  ksort($tags);
  return $tags;
}

function return_i18n_search_results($tags=null, $words=null, $first=0, $max=10, $excerptlength=30) {
  $results = i18n_search($tags, $words);
  $count = count($results);
  if ($max > 0) $results = array_slice($results, $first, $max);
  if ($results && count($results) > 0) {
    foreach ($results as &$result) {
      $pagedata = getXML(GSDATAPAGESPATH . $result['slug'] . '.xml');
      $content = stripslashes(html_entity_decode($pagedata->content, ENT_QUOTES, 'UTF-8'));    
      if ($excerptlength > 0) {
        // Get first "window" words
        $excerpt = preg_split("/\s+/", strip_tags($content), $excerptlength+1);
        if (count($excerpt) > $excerptlength) { 
          array_pop($excerpt); array_push($excerpt, '...'); 
          $result['more'] = true;
        } else {
          $result['more'] = false;
        }
        $result['excerpt'] = implode(' ', $excerpt);
      } else if ($excerptlength < 0) {
        $result['content'] = $content; 
      } else {
        $result['excerpt'] = '';
      }
      $result['title'] = stripslashes(html_entity_decode($pagedata->title, ENT_QUOTES, 'UTF-8'));
      $result['parent'] = '' . $pagedata->parent;
      $result['tags'] = preg_split("/\s*,\s*/", trim($pagedata->metak));
    }
  }
  return array('totalCount' => $count, 'first' => $first, 'results' => $results);
}

function i18n_compare_search_results($a, $b) {
  if ($a['score'] == $b['score']) {
    $r = $b['date'] - $a['date'];
    return $r != 0 ? $r : strcmp($a['url'], $b['url']);
  } else {
    return $b['score'] - $a['score'];
  }
}

function i18n_search_params($params) {
  global $language, $LANG;
  if (file_exists(GSDATAOTHERPATH.I18N_SEARCH_SETTINGS_FILE)) {
    $data = getXML(GSDATAOTHERPATH.I18N_SEARCH_SETTINGS_FILE);
    if ($data) foreach ($data->children() as $child) {
      if (!array_key_exists($child->getName(), $params)) $params[$child->getName()] = (string) $child;
    }
  }
  if (isset($language)) i18n_search_merge_texts($language, $params);
  if (isset($LANG)) i18n_search_merge_texts(substr($language,0,2), $params);
  i18n_search_merge_texts('en', $params);
  return $params;  
}

function i18n_search_merge_texts($lang, &$params) { 
  $i18n = array();
  if (!file_exists(GSPLUGINPATH.'i18n_search/lang/'.$lang.'.php')) return false;
  @include(GSPLUGINPATH.'i18n_search/lang/'.$lang.'.php'); 
  if (count($i18n) > 0) foreach ($i18n as $code => $text) {
    if (!array_key_exists($code, $params)) $params[$code] = $text;
  }
  return true;
}


function i18n_search_replace($content) {
  return preg_replace_callback("/(<p>\s*)?\(%\s*(searchform|searchresults|tags)(\s+(?:%[^%\)]|[^%])+)?\s*%\)(\s*<\/p>)?/", 'i18n_search_replace_match',$content);
}

function i18n_search_replace_match($match) {
  global $args;
  $replacement = '';
  if ($match[1] && (!isset($match[4]) || !$match[4])) $replacement .= $match[1];
  $function = $match[2];
  $params = array();
  $paramstr = html_entity_decode(trim($match[3]), ENT_QUOTES, 'UTF-8');
  while (preg_match('/^([a-zA-Z][a-zA-Z_-]*)[:=]([^"\'\s]*|"[^"]*"|\'[^\']*\')(?:\s|$)/', $paramstr, $pmatch)) {
    $key = $pmatch[1];
    $value = trim($pmatch[2]);
    if (substr($value,0,1) == '"' || substr($value,0,1) == "'") $value = substr($value,1,strlen($value)-2);
    $params[$key] = $value;
    $paramstr = substr($paramstr, strlen($pmatch[0]));
  }
  ob_start();
  if ($function == 'searchform') {
    get_i18n_search_form($params);
  } else if ($function == 'searchresults') {
    get_i18n_search_results($params);
  } else if ($function == 'tags') {
    get_i18n_tags($params);
  }
  $replacement .= ob_get_contents();
  ob_end_clean();
  if (!$match[1] && isset($match[4]) && $match[4]) $replacement .= $match[4];
  return $replacement;
}

function i18n_search_configure() {
  include(GSPLUGINPATH.'i18n_search/configure.php');
}

function get_i18n_search_form($params=null) {
  $params = i18n_search_params(is_array($params) ? $params : array());
  include(GSPLUGINPATH.'i18n_search/searchform.php');
}

function get_i18n_search_results($params=null) {
  $params = i18n_search_params(is_array($params) ? $params : array());
  include(GSPLUGINPATH.'i18n_search/searchresults.php');
}

function get_i18n_tags($params=null) {
  $params = i18n_search_params(is_array($params) ? $params : array());
  $minTagSizePercent = array_key_exists('minTagSize',$params) ? (int) $params['minTagSize'] : I18N_MIN_TAG_SIZE;
  $maxTaxSizePercent = array_key_exists('maxTagSize',$params) ? (int) $params['maxTagSize'] : I18N_MAX_TAG_SIZE;
  get_i18n_tags_impl($minTagSizePercent, $maxTaxSizePercent);
}

function get_i18n_tags_impl($minPercent=100, $maxPercent=250) {
  include(GSPLUGINPATH.'i18n_search/tags.php');
}


